from random import *
import os
from direct.task import Task
from direct.showbase.DirectObject import DirectObject
from pandac.PandaModules import *

#load cfg file
CFG = 'data/config/'
config = {}
file = open(CFG+"user.cfg", 'r')        
while 1:
    string = file.readline()
    if (string == ""):
        break
    part = string.rsplit('=')
    config[part[0].strip()] = part[1].strip()
file.close()
props = WindowProperties()
loadPrcFileData("", "fullscreen "+config['fullscreen'])
loadPrcFileData("", "show-frame-rate-meter "+config['showfps'])
import direct.directbase.DirectStart
props.setSize(int(config['resx']), int(config['resy']))
base.win.requestProperties(props)
base.disableMouse()

from simulation.world import World
from simulation.ship import Ship
from client.navigation import Navigation, Spectator
from ai.simple import *
from ai.complex import *
from client.loader import ShipLoader
from client.skybox import SkyBox
from client.hud import Hud
from client.shaders import GlowCamera
from client.menu import Menu
#base.setFrameRateMeter(True)

class Game(DirectObject):
    
    WATCH_AI = False
    
    def __init__(self):
        self.config = config
        self.availableMaps = ['IADM_Twistroid2', 'Derelict2']
        self.availableShips = ['sword']
        self.menu = Menu(self)
        
        
    def startGame(self, mapName="IADM_Twistroid2", playerModel='sword'):
        pl =  base.cam.node().getLens()
        pl.setFar(10000)
        pl.setNear(1)
        pl.setFov(90)
        base.cam.node().setLens(pl)
        
        #self.glowCamera = GlowCamera(self)
        #self.testingStuff()
        #taskMgr.add(self.glowCamera.task, 'glow camera task')
        

        
        self.skyBox = SkyBox()
        self.skyBox.load("data/skyboxes/iaskb4_000#.png")
        self.shipLoder = ShipLoader()
        self.shipLoder.cacheModel('sword')
        if playerModel != 'sword':
            self.shipLoder.cacheModel(playerModel)
        
        self.world = World(self)

        #mapName = "IADM_Twistroid2"#"Derelict2" 
        self.world.loadMap(mapName)
        points = __import__(mapName).points
        self.aiGraph = Graph(points)
        
        
        if self.WATCH_AI != True :
            self.navigation = Navigation(self)
            self.plyerShip = Ship(self.shipLoder.loadShip('sword'))
            self.world.addShip(self.plyerShip)        
            self.navigation.takeShip(self.plyerShip)
        
        for aiShip in range(4):
            aiShip = Ship(self.shipLoder.loadShip('sword'))
            self.world.addShip(aiShip)        
            Stalker(self).takeShip(aiShip)
            
        
            
        if self.WATCH_AI == True :
            self.navigation = Spectator(self)
            self.navigation.takeShip(aiShip)
            self.plyerShip = aiShip
            self.aiGraph.generate()
            
        self.hud = Hud(self.plyerShip,self.world)
        taskMgr.add(self.hud.update, 'hud update task')    
        taskMgr.add(self.world.simulate, 'world simulate task')
        
        
        self.accept( "space" , self.mark )
        self.accept( "enter", self.record )
        
        
    def testingStuff(self):
        
        # load the object with the disturbance
        self.disturbanceObject = loader.loadModel("data/testing/cube.x")
        self.disturbanceObject.setScale(100)
        self.disturbanceObject.setPos(1000,1000,1000)
        
        self.disturbanceObject.reparentTo(render)
        #self.disturbanceObject.setTransparency(True)
        #self.disturbanceObject.setAttrib(ColorBlendAttrib.make(ColorBlendAttrib.MAdd))
        self.disturbanceObject.setShader( Shader.load( "data/shaders/disturbance.sha") )
        self.disturbanceObject.hide(BitMask32.bit(2))
        # Textures
        tex1 = loader.loadTexture( 'data/shaders/water.png' )
        ts1 = TextureStage( 'distortion' )
        ts1.setSort(1)
        self.disturbanceObject.setTexture( ts1, tex1 )
        tex0 = self.glowCamera.screenTexture
        ts0 = TextureStage( 'reflection' )
        ts0.setSort(0)
        self.disturbanceObject.setTexture( ts0, tex0 ) 
        
        
    def record(self):
        base.movie('/home/desert/Python_Panda_Pygame/shared_projects/lens-flare/movies', duration = 60.0, fps = 30, format='bmp') 
   
    def mark(self):
        pos = self.plyerShip.node.getPos()
        print "\t\tPoint3(%f,%f,%f),"%(pos[0],pos[1],pos[2])   
   
    def run(self):
        #render.ls()
        run()