from direct.showbase import DirectObject
from pandac.PandaModules import *
from direct.task import Task
import math, sys, random, os
from direct.interval.IntervalGlobal import *

from direct.gui.DirectGui import *

CFG = 'data/config/'
props = WindowProperties()

class Menu:
    def __init__(self, base):
        self.base = base
        self.mainMenu()
        self.currentMap = self.base.availableMaps[0]
        self.currentShip = self.base.availableShips[0]
        
    def mainMenu(self):
        self.color = Vec4(1,1,1,1)
        offset=-0.2
        self.start = DirectButton(text = ("Start Game","<Start Game>","< Start Game >", "|Start Game|"), command=self.switchFromMainToStart, scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.start.setPos(0,0,offset+0.55)
        
        self.player = DirectButton(text = ("Player Settings","<Player Settings>","< Player Settings >","|Player Settings|"), command=self.switchFromMainToPlayer, scale=0.05, state=1,  text_fg=self.color, relief=None)        
        self.player.setPos(0,0,offset+0.35)
        
        self.display = DirectButton(text = ("Display Settings","<Display Settings>","< Display Settings >","|Display Settings|"), command=self.switchFromMainToDisplay, scale=0.05, state=1,  text_fg=self.color, relief=None)        
        self.display.setPos(0,0,offset+0.25)
        
        self.control = DirectButton(text = ("Control Settings","<Control Settings>","< Control Settings >","|Control Settings|"), command=self.switchFromMainToControls, scale=0.05, state=0, text_fg=self.color, relief=None)        
        self.control.setPos(0,0,offset+0.15)
        
        self.credits = DirectButton(text = ("Credits","<Credits>","< Credits >", "|Credits|"), command=self.switchFromMainToCredits, scale=0.05, state=0, text_fg=self.color, relief=None)        
        self.credits.setPos(0,0,offset-0.05)
        
        #self.sound = DirectButton(text = ("Sound Settings","<Sound Settings>","< Sound Settings >","|Sound Settings|"), command=self.switchFromMainToAudio, scale=0.05, state=1, text_font=globalFx['menufont'], text_fg=self.color, relief=None)        
        #self.sound.setPos(0,0,offset+0.05)
        
        self.exit = DirectButton(text = ("Exit", "<Exit>", "< Exit >", "|Exit|"), command=self.exitFunc, scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.exit.setPos(0,0,offset-0.15)
    
    def unloadMain(self):
        self.start.destroy()
        self.player.destroy()
        self.display.destroy()
        self.control.destroy()
        self.credits.destroy()
        #self.sound.destroy()
        self.exit.destroy()
        
    def loadStart(self):
        inititem = 0
        offset=-0.2
        self.mapListLabel = DirectLabel(text = ("Select Map:"), scale=0.05, text_fg=self.color, relief=None)
        self.mapListLabel.setPos(-0.7,0,offset+1)
        self.mapList = DirectOptionMenu(text="options", scale=0.04,
                                        items=self.base.availableMaps,
                                        initialitem=inititem,highlightColor=(0.65,0.65,0.65,1),
                                        command=self.changeCurrentMap, text_fg=self.color, relief=None)
        self.mapList.setPos(-0.5,0,offset+1)
        
        self.dialogFrame = DirectScrolledFrame(scale=0.35,frameSize=(-2.5,2.5,-1,1), canvasSize=(-2.5, 2.5, -1, 1),frameColor=(0,0,0,0.3), 
                    scrollBarWidth=0.1,verticalScroll_frameColor=(0,0,0.5,0.5),verticalScroll_value=0.25,
                    verticalScroll_incButton_geom_scale=0.07,verticalScroll_decButton_geom_scale=0.07)
        self.dialogNode = self.dialogFrame.getCanvas()
        self.mapDescription = DirectLabel(text = ("Map Description and picture for "+self.base.availableMaps[0]),
                                            scale=0.1, text_fg=self.color, relief=None)
        self.mapDescription.reparentTo(self.dialogNode)
        self.mapDescription.setPos(-0.7,0,offset+1)
        
        self.start = DirectButton(text = ("Start","<Start>","< Start >", "|Start|"), command=self.startGame, 
                                    scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.start.setPos(0.7,0,offset-0.5)
        self.back = DirectButton(text = ("Back","<Back>","< Back >", "|Back|"), command=self.switchFromStartToMain, 
                                    scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.back.setPos(-0.7,0,offset-0.5)
        
    def loadPlayer(self):
        inititem = 0
        offset=-0.2
        self.shipListLabel = DirectLabel(text = ("Select Ship:"), scale=0.05, text_fg=self.color, relief=None)
        self.shipListLabel.setPos(-0.7,0,offset+1)
        self.shipList = DirectOptionMenu(text="options", scale=0.04,
                                        items=self.base.availableShips,
                                        initialitem=inititem,highlightColor=(0.65,0.65,0.65,1),
                                        command=self.changeCurrentShip, text_fg=self.color, relief=None)
        self.shipList.setPos(-0.5,0,offset+1)
        self.nameLabel = DirectLabel(text = ("Player Name:"), scale=0.05, text_fg=self.color, relief=None)
        self.nameLabel.setPos(-0.7,0,offset+0.7)
        self.name = DirectEntry(text = "" ,scale=.05,command=self.changePlayerName, 
                                initialText=self.base.config['playerName'], numLines = 1,focus=0,
                                text_fg=self.color, relief=None)
        self.name.setPos(-0.5,0,offset+0.7)
        
        self.dialogFrame = DirectScrolledFrame(scale=0.35,frameSize=(-2.5,2.5,-1,1), canvasSize=(-2.5, 2.5, -1, 1),frameColor=(0,0,0,0.3), 
                    scrollBarWidth=0.1,verticalScroll_frameColor=(0,0,0.5,0.5),verticalScroll_value=0.25,
                    verticalScroll_incButton_geom_scale=0.07,verticalScroll_decButton_geom_scale=0.07)
        self.dialogNode = self.dialogFrame.getCanvas()
        self.shipDescription = DirectLabel(text = ("Ship Picture and Description for "+self.currentShip),
                                            scale=0.1, text_fg=self.color, relief=None)
        self.shipDescription.reparentTo(self.dialogNode)
        self.shipDescription.setPos(-0.7,0,offset+1)
        
        self.back = DirectButton(text = ("Back","<Back>","< Back >", "|Back|"), command=self.switchFromPlayerToMain, 
                                    scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.back.setPos(-0.7,0,offset-0.5)
        self.save = DirectButton(text = ("Save","<Save>","< Save >", "|Save|"), command=self.saveSettings, 
                                    scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.save.setPos(0.7,0,offset-0.5)
        
    def loadDisplay(self):
        inititem=0
        resx = self.base.config['resx']
        initfsc = int(self.base.config['fullscreen'])
        initfps = int(self.base.config['showfps'])
        if resx == '640': inititem = 0
        if resx == '800': inititem = 1
        if resx == '1024': inititem = 2
        if resx == '1280': inititem = 3
        offset=-0.2
        self.resolutionListLabel = DirectLabel(text = ("Resolution:"), scale=0.05, text_fg=self.color, relief=None)
        self.resolutionListLabel.setPos(-0.7,0,offset+1)
        self.resolutionList = DirectOptionMenu(text="options", scale=0.04,
                                        items=['640x480', '800x600', '1024x768', '1280x1024'],
                                        initialitem=inititem,highlightColor=(0.65,0.65,0.65,1),
                                        command=self.changeResolution, text_fg=self.color, relief=None)
        self.resolutionList.setPos(-0.5,0,offset+1)
        
        self.fullscreen = DirectCheckButton(indicatorValue=initfsc,text = "Fullscreen (takes effect after restart)",
                                            scale=0.05,command=self.changeFullscreen,text_fg=self.color, relief=None)
        self.fullscreen.setPos(0,0,offset+0.45)
        self.showfps = DirectCheckButton(indicatorValue=initfps,text = "Show FPS" ,scale=0.05,
                                            command=self.changeShowfps,text_fg=self.color, relief=None)
        self.showfps.setPos(0,0,offset+0.35)
        
        self.back = DirectButton(text = ("Back","<Back>","< Back >", "|Back|"), command=self.switchFromDisplayToMain, 
                                    scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.back.setPos(-0.7,0,offset-0.5)
        self.save = DirectButton(text = ("Save","<Save>","< Save >", "|Save|"), command=self.saveSettings, 
                                    scale=0.05, state=1, text_fg=self.color, relief=None)        
        self.save.setPos(0.7,0,offset-0.5)
        
    def unloadDisplay(self):
        self.resolutionListLabel.destroy()
        self.resolutionList.destroy()
        self.fullscreen.destroy()
        self.showfps.destroy()
        self.back.destroy()
        self.save.destroy()
        
    def unloadPlayer(self):
        self.shipListLabel.destroy()
        self.shipList.destroy()
        self.nameLabel.destroy()
        self.name.destroy()
        self.dialogFrame.destroy()
        self.back.destroy()
        self.save.destroy()
        
    def unloadStart(self):
        self.mapListLabel.destroy()
        self.mapList.destroy()
        self.start.destroy()
        self.back.destroy()
        self.dialogFrame.destroy()
        
    def startGame(self):
        self.unloadStart()
        self.base.startGame(self.currentMap, self.currentShip)
        
    def switchFromDisplayToMain(self):
        self.unloadDisplay()
        self.mainMenu()
        
    def switchFromPlayerToMain(self):
        self.unloadPlayer()
        self.mainMenu()
    
    def switchFromMainToStart(self):
        self.unloadMain()
        self.loadStart()
        
    def switchFromStartToMain(self):
        self.unloadStart()
        self.mainMenu()
        
    def switchFromMainToPlayer(self):
        self.unloadMain()
        self.loadPlayer()
    
    def switchFromMainToDisplay(self):
        self.unloadMain()
        self.loadDisplay()
        
    def switchFromMainToControls(self):
        self.unloadMain()
        
    def switchFromMainToCredits(self):
        self.unloadMain()
    
    def changeFullscreen(self,status):
        if (status == 1):
            self.base.config['fullscreen'] = "1"
        if (status == 0):
            self.base.config['fullscreen'] = "0"
            
    def changeShowfps(self,status):
        if (status == 1):
            self.base.config['showfps'] = "1"
            base.setFrameRateMeter(True)
        if (status == 0):
            base.setFrameRateMeter(False)
            self.base.config['showfps'] = "0"
        
    def changeResolution(self,resolution):
        if (resolution == "1280x1024"):
            self.base.config['resx'] = "1280"
            self.base.config['resy'] = "1024"
        if (resolution == "1024x768"):
            self.base.config['resx'] = "1024"
            self.base.config['resy'] = "768"
        if (resolution == "800x600"):
            self.base.config['resx'] = "800"
            self.base.config['resy'] = "600"
        if (resolution == "640x480"):
            self.base.config['resx'] = "640"
            self.base.config['resy'] = "480"
        props.setSize(int(self.base.config['resx']), int(self.base.config['resy']))
        base.win.requestProperties(props)
        
    def changePlayerName(self, name):
        if name != "":
            self.base.config['playerName'] = name
        
    def changeCurrentShip(self, ship):
        self.currentShip = ship
        offset=-0.2
        self.dialogFrame.destroy()
        self.dialogFrame = DirectScrolledFrame(scale=0.35,frameSize=(-2.5,2.5,-1,1), canvasSize=(-2.5, 2.5, -1, 1),frameColor=(0,0,0,0.3), 
                    scrollBarWidth=0.1,verticalScroll_frameColor=(0,0,0.5,0.5),verticalScroll_value=0.25,
                    verticalScroll_incButton_geom_scale=0.07,verticalScroll_decButton_geom_scale=0.07)
        self.dialogNode = self.dialogFrame.getCanvas()
        self.shipDescription = DirectLabel(text = ("Ship Picture and Description for "+self.currentShip),
                                            scale=0.1, text_fg=self.color, relief=None)
        self.shipDescription.reparentTo(self.dialogNode)
        self.shipDescription.setPos(-0.7,0,offset+1)
        
    def changeCurrentMap(self, map):
        self.currentMap = map
        offset=-0.2
        self.dialogFrame.destroy()
        self.dialogFrame = DirectScrolledFrame(scale=0.35,frameSize=(-2.5,2.5,-1,1), canvasSize=(-2.5, 2.5, -1, 1),frameColor=(0,0,0,0.3), 
                    scrollBarWidth=0.1,verticalScroll_frameColor=(0,0,0.5,0.5),verticalScroll_value=0.25,
                    verticalScroll_incButton_geom_scale=0.07,verticalScroll_decButton_geom_scale=0.07)
        self.dialogNode = self.dialogFrame.getCanvas()
        self.mapDescription = DirectLabel(text = ("Map Description and picture for "+self.currentMap),
                                            scale=0.1, text_fg=self.color, relief=None)
        self.mapDescription.reparentTo(self.dialogNode)
        self.mapDescription.setPos(-0.7,0,offset+1)
        
    def saveSettings(self):
        file = open(CFG+"user.cfg", 'w')    
        for key in self.base.config: 
            string = key+"="+self.base.config[key]+"\n"
            file.write(string)
        file.close()
            
    def exitFunc(self):
        self.unloadMain()
        
        self.dialogFrame = DirectScrolledFrame(scale=0.35,frameSize=(-2.5,2.5,-1,1), canvasSize=(-2.5, 2.5, -1, 1),frameColor=(0,0,0,0.3), 
                    scrollBarWidth=0.1,verticalScroll_frameColor=(0,0,0.5,0.5),verticalScroll_value=0.25,
                    verticalScroll_incButton_geom_scale=0.07,verticalScroll_decButton_geom_scale=0.07)
        self.dialogNode = self.dialogFrame.getCanvas()
        offset=-0.3
        
        self.text = DirectLabel(text = ("Right, like YOU have anything better to do!"), scale=0.15, text_fg=self.color, relief=None)
        self.text.reparentTo(self.dialogNode)
        self.yes = DirectButton(text = ("yes", "<yes>", "< yes >", "|yes|"), command=self.dialogFuncYes, scale=0.15, state=1, text_fg=self.color, relief=None)
        self.yes.reparentTo(self.dialogNode)
        self.yes.setPos(-0.5,0,offset-0.5)
        self.no = DirectButton(text = ("no", "<no>", "< no >", "|no|"), scale=0.15, command=self.dialogFuncNo, text_fg=self.color, relief=None)
        self.no.reparentTo(self.dialogNode)
        self.no.setPos(0.5,0,offset-0.5)
        
                    
    def dialogFuncYes(self):
        print "They'll be waiting for you"
        sys.exit()
    
    def dialogFuncNo(self):
        self.dialogFrame.destroy()
  
        self.mainMenu()