from pandac.PandaModules import *
from direct.task import Task

class SkyBox: 
 
    def __init__(self):    
        taskMgr.add(self.setSkyBoxTask,'set sky box')
        self.node = NodePath("skybox")
        self.node.reparentTo(render)
        self.node.setScale(10)
        cm = CardMaker("rightPane")
        cm.setFrame(10,-10,-10,10)
        self.rightPane = self.node.attachNewNode(cm.generate())
        self.rightPane.setPos(10,0,0)
        self.rightPane.setH(-90)
        self.rightPane.setR(270)
        cm = CardMaker("leftPane")
        cm.setFrame(10,-10,-10,10)
        self.leftPane = self.node.attachNewNode(cm.generate())
        self.leftPane.setPos(-10,0,0)
        self.leftPane.setH(90)
        self.leftPane.setR(90)
        cm = CardMaker("frontPane")
        cm.setFrame(10,-10,-10,10)
        self.frontPane = self.node.attachNewNode(cm.generate())
        self.frontPane.setPos(0,-10,0)
        self.frontPane.setH(180)
        cm = CardMaker("backPane")
        cm.setFrame(10,-10,-10,10)
        self.backPane = self.node.attachNewNode(cm.generate())
        self.backPane.setPos(0,10,0)
        self.backPane.setR(180)
        cm = CardMaker("topPane")
        cm.setFrame(10,-10,-10,10)
        self.topPane = self.node.attachNewNode(cm.generate())
        self.topPane.setPos(0,0,10)
        self.topPane.setP(90)
        self.topPane.setH(180)
        cm = CardMaker("bottomPane")
        cm.setFrame(10,-10,-10,10)
        self.bottomPane = self.node.attachNewNode(cm.generate())
        self.bottomPane.setPos(0,0,-10)
        self.bottomPane.setP(-90)
        self.node.setTwoSided(True)
        self.node.setDepthWrite(False)
        self.node.hide(BitMask32(2))
        self.node.setBin("background",0)
        self.node.setAttrib(LightAttrib.makeAllOff())
       
    def loadClampedTexture(self,name):
        texture = loader.loadTexture(name)
        texture.setWrapU(Texture.WMClamp)
        texture.setWrapV(Texture.WMClamp)
        return texture

    def load(self,textures,rotatoin=Vec3(0,0,0)):
        """ set up a sky boc with the new texture """
        self.rightPane.setTexture(self.loadClampedTexture(textures.replace("#","0"))) 
        self.leftPane.setTexture(self.loadClampedTexture(textures.replace("#","1"))) 
        self.frontPane.setTexture(self.loadClampedTexture(textures.replace("#","2"))) 
        self.backPane.setTexture(self.loadClampedTexture(textures.replace("#","3"))) 
        self.topPane.setTexture(self.loadClampedTexture(textures.replace("#","4"))) 
        self.bottomPane.setTexture(self.loadClampedTexture(textures.replace("#","5"))) 
        self.node.setHpr(rotatoin)
       
    def setSkyBoxTask(self,task):
        self.node.setPos(base.camera.getPos())
        return Task.cont 