from random import *
import os
from direct.task import Task
from direct.showbase.DirectObject import DirectObject
import direct.directbase.DirectStart
from pandac.PandaModules import *
import math

class CollisionDetection(DirectObject):
    def __init__(self,world):
        self.world = world
        self.traverser = CollisionTraverser('collisionTraverser')
        self.traverser.setRespectPrevTransform(True)
        if self.world.showCollisions: self.traverser.showCollisions(render)
        base.cTrav = self.traverser
        self.traverser.traverse(render)
        self.handler = CollisionHandlerEvent()
        self.handler.addInPattern('into')
        self.handler.addInPattern('again')
        self.accept('into', self.findIntoNodes)
        self.accept('again', self.findIntoNodes)
        
    def addFromNodePath(self, nodePath):
        self.traverser.addCollider(nodePath, self.handler)
        
    def removeFromNodePath(self, nodePath):
        self.traverser.removeCollider(nodePath)
        
    def findIntoNodes(self, colEntry):
        fromNode = colEntry.getFromNodePath()
        intoNode = colEntry.getIntoNodePath()
    
        self.normal = colEntry.getSurfaceNormal(render)
        self.point = colEntry.getSurfacePoint(render)
        intoType = intoNode.getNetTag('collisionType')
        fromType = fromNode.getNetTag('collisionType')
        if (intoType == 'brush'): self.handleBrushCollision(intoNode, fromNode, fromType)
        if (intoType == 'shield'): self.handleShieldCollision(intoNode, fromNode, fromType)
        if (intoType == 'weapon' and  fromType == 'shield'): self.handleShieldCollision(fromNode, intoNode, intoType)
    
    def handleBrushCollision(self, intoNode, fromNode, fromType):
        if fromType == "shield":
            ship = self.world.ships[int(fromNode.getNetTag('id'))]
            
            shipNode = fromNode.getParent()
            shipNode.setPos(shipNode.getPos()+self.normal*ship.speed*10)
            
        if fromType == "weapon":
           try:
                projectile = self.world.projectiles[int(fromNode.getNetTag('id'))]
                self.world.removeProjectile(projectile)
           except KeyError,e:
              pass # projectiles is already gone
            
    def handleShieldCollision(self,intoNode, fromNode, fromType):
        if fromType == 'shield':
            print "shield-shield"
            return # do we even need to hande ship to ship collisions ?
            ship = fromNode.getParent()
            ship2 = intoNode.getParent()
            
            bounceVec = -(ship.getPos() - ship2.getPos())*.1
            ship.setPos(ship.getPos()+bounceVec)
            ship2.setPos(ship2.getPos()-bounceVec)
            
        if fromType == 'weapon':            
            try:
                projectile = self.world.projectiles[int(fromNode.getNetTag('id'))]
                ship   = self.world.ships[int(intoNode.getNetTag('id'))]
                if projectile.launched != ship:
                    ship.structure -= projectile.damage
                    self.world.removeProjectile(projectile)
                    if ship.structure < 0 :
                        self.world.removeShip(ship)
            except KeyError:
                pass # projectiles or ships are already gone
                        
    
class CollisionOptimizer:
    
    
    def optimize(self,node):
        self.object = node
        self.faces = {}
        self.processGeomNode(self.object.node())
        self.doOptimization()
        return self.generate()
      
    def process(self):
        geomNodeCollection = self.object.findAllMatches('+GeomNode')
        print "coll",geomNodeCollection.asList()
        for nodePath in geomNodeCollection.asList():
            geomNode = nodePath.node()
            self.processGeomNode(geomNode)
          
    def processGeomNode(self,geomNode):    
         
        for i in range(geomNode.getNumGeoms()):
            geom = geomNode.getGeom(i)
            state = geomNode.getGeomState(i)
            self.processGeom(geom)
                
    def processGeom(self,geom):
        vdata = geom.getVertexData()
        for i in range(geom.getNumPrimitives()):            
            prim = geom.getPrimitive(i)
            self.processPrimitive(prim, vdata)  
            
    def processPrimitive(self,prim, vdata):
        vertex = GeomVertexReader(vdata, 'vertex')        
        prim = prim.decompose()
        for p in range(prim.getNumPrimitives()):
            s = prim.getPrimitiveStart(p)
            e = prim.getPrimitiveEnd(p)
            face = []
            for i in range(s, e):
                vi = prim.getVertex(i)
                vertex.setRow(vi)
                v = vertex.getData3f()
                face.append(v)
            self.faces[tuple(face)] = True  
                        
    def doOptimization(self):  #, destNode, cellSize, collide=False):
        bounds = self.object.getBounds()
        self.center = bounds.getCenter()
        radius = bounds.getRadius()
        # Loop on the vertices determine the bounding box
        self.minBB = self.center - Vec3(radius,radius,radius) 
        self.maxBB = self.center + Vec3(radius,radius,radius) 
        self.max = 32
        
        
        self.s = radius*2/self.max
        s = self.s
        
        
        stx,sty,stz = self.minBB[0],self.minBB[1],self.minBB[2]
        
        self.cels = [[[ [] for i in range(self.max+1) ] for n in range(self.max+1) ] for j in range(self.max+1)]
        
        print self.minBB , s
        for face in self.faces.keys():
            center = (face[0]+face[1]+face[2])/3 - self.minBB          
            cx = int(center[0]/s)
            cy = int(center[1]/s)
            cz = int(center[2]/s)
            #print cx,cy,cz
            self.cels[cx][cy][cz].append(face)
      
      
    def generateStupid(self):
        print "generating stupid"
        s = self.s
        rootNode = NodePath("root optimized node")
        for sx in range(0,self.max):
            for sy in range(0,self.max):
                for sz in range(0,self.max):
                    faces = self.cels[sx][sy][sz]
                    if faces != []:
                        print "faces!"
                        #self.addf += len(faces)
                        #self.addg += 1
                        blockCenter = Vec3(sx*s,sy*s,sz*s) + self.minBB        
                                
                        #print len(faces)           
                        geomnode = self.generateFromFaces(faces,blockCenter)

                        nodepath = NodePath(str(blockCenter))
                        nodepath.attachNewNode(geomnode)
                        nodepath.setCollideMask(BitMask32.bit(1))
                        nodepath.setTag('collisionType', 'brush')
                        nodepath.setPos(blockCenter)
                        #nodepath.setHpr(213,23,21)
                        
                        nodepath.reparentTo(rootNode) 
              
        #print "faces per geom" , float(self.addf)/self.addg
        #rootNode.flattenLight()
        #rootNode.flattenMedium()
        rootNode.flattenStrong()
        
        return rootNode
      
    def generate(self):
        self.addf = 0
        self.addg = 0
        blockSize = self.max
        sx = 0
        sy = 0 
        sz = 0
        rootNode = NodePath("root optimized node")
        h = blockSize/2
        for i in (0,h):
            for j in (0,h):
                for k in (0,h):
                    node = self.generateRec(blockSize/2,sx+i,sy+j,sz+k,self.center)
                    node.reparentTo(rootNode)      
        print "faces per geom" , float(self.addf)/self.addg
        #rootNode.flattenLight()
        #rootNode.flattenMedium()
        rootNode.flattenStrong()
        
        return rootNode

    def generateRec(self,blockSize,sx,sy,sz,rootCetner):
        s = self.s
        rootNode = None        
        blockCenter = Vec3(sx*s,sy*s,sz*s) - self.minBB        
        
        h = blockSize/2
        if blockSize > 1:
            for i in (0,h):
                for j in (0,h):
                    for k in (0,h):
                        node = self.generateRec(blockSize/2,sx+i,sy+j,sz+k,blockCenter)
                        if node != None:
                            if rootNode == None:
                                rootNode = NodePath("part-%f%f%f"%(sx,sy,sz))
                                rootNode.setPos(blockCenter-rootCetner)
                            node.reparentTo(rootNode)
                            
                        
        else:
            #print sx,sy,sz
            faces = self.cels[sx][sy][sz]
            if faces != []:
                self.addf += len(faces)
                self.addg += 1
                 
                                
                #print len(faces)           
                geomnode = self.generateFromFaces(faces,blockCenter)

                nodepath = NodePath(str(blockCenter))
                nodepath.attachNewNode(geomnode)
                nodepath.setCollideMask(BitMask32.bit(1))
                nodepath.setTag('collisionType', 'brush')
                nodepath.setPos(blockCenter-rootCetner)
                #nodepath.setHpr(213,23,21)
                
                
                if rootNode == None:
                    rootNode = NodePath("part-%f%f%f"%(sx,sy,sz))
                    rootNode.setPos(blockCenter-rootCetner)
                nodepath.reparentTo(rootNode)                
                
        return rootNode

    def generateFromFaces(self,faces,offset): 
        vdata = GeomVertexData('new geom', GeomVertexFormat.getV3() , Geom.UHStatic)
        vertex = GeomVertexWriter(vdata, 'vertex')
        number = 0
        for face in  faces:
            v1,v2,v3 = face
            
            #assert (v1+offset).length() < self.s
            print v1,offset,self.s,v1+offset
            
            vertex.addData3f(v1-offset)
            vertex.addData3f(v2-offset)
            vertex.addData3f(v3-offset)  
            number+=1
        prim = GeomTriangles(Geom.UHStatic)
        for n in range(number):
            prim.addVertices(n*3,n*3+1,n*3+2)  
        prim.closePrimitive()
        geom = Geom(vdata)
        geom.addPrimitive(prim)
        geomnode = GeomNode('gnode')
        geomnode.addGeom(geom)
        return geomnode