from random import *
import os
from direct.task import Task
from direct.showbase.DirectObject import DirectObject
import direct.directbase.DirectStart
from pandac.PandaModules import *
import math

class CollisionDetection(DirectObject):
    def __init__(self,world):
        self.world = world
        
        self.queue = CollisionHandlerQueue()
        self.traverser = CollisionTraverser()
       
        
        if self.world.showCollisions: self.traverser.showCollisions(render)
        
        taskMgr.add(self.collisionTask, 'collision task')
        
        
    def addFromNodePath(self, nodePath):
        self.traverser.addCollider(nodePath, self.queue)
        
    def removeFromNodePath(self, nodePath):
        self.traverser.removeCollider(nodePath)
        
    def collisionTask(self,task):
        
        self.traverser.setRespectPrevTransform(True)
        self.traverser.traverse(render)

        entries = self.queue.getNumEntries()
        
        for i in range(entries):
            entry = self.queue.getEntry(i)
            #print entry
            self.findIntoNodes(entry)
            
        return Task.cont
        
        
    def findIntoNodes(self, colEntry):
        fromNode = colEntry.getFromNodePath()
        intoNode = colEntry.getIntoNodePath()
        self.normal = colEntry.getSurfaceNormal(render)
        self.point = colEntry.getSurfacePoint(render)
        intoType = intoNode.getNetTag('collisionType')
        fromType = fromNode.getNetTag('collisionType')
        if (intoType == 'brush'): self.handleBrushCollision(intoNode, fromNode, fromType)
        if (intoType == 'shield'): self.handleShieldCollision(intoNode, fromNode, fromType)
        if (intoType == 'weapon' and  fromType == 'shield'): self.handleShieldCollision(fromNode, intoNode, intoType)
    
    def handleBrushCollision(self, intoNode, fromNode, fromType):
        if fromType == "shield":
            ship = self.world.ships[int(fromNode.getNetTag('id'))]
            shipNode = fromNode.getParent()
            
            direction = self.point-shipNode.getPos()
            direction.normalize()
            self.normal.normalize()
            if (self.normal+direction).length() < 1 :                
                shipNode.setPos(shipNode.getPos()+self.normal*abs(ship.speed*.1))
            else:
                shipNode.setPos(shipNode.getPos()-self.normal*abs(ship.speed*.1))
        if fromType == "weapon":
           try:
                projectile = self.world.projectiles[int(fromNode.getNetTag('id'))]
                self.world.hitWorld(projectile)
                self.world.removeProjectile(projectile)
           except KeyError,e:
              pass # projectiles is already gone
            
    def handleShieldCollision(self,intoNode, fromNode, fromType):
        if fromType == 'shield':
            #print "shield-shield"
            return
             # do we even need to hande ship to ship collisions ?
            ship = fromNode.getParent()
            ship2 = intoNode.getParent()            
            ship.setPos(ship,Vec3(0,0,1))
            ship2.setPos(ship2,Vec3(0,0,-1))
            
        if fromType == 'weapon':            
            try:
                projectile = self.world.projectiles[int(fromNode.getNetTag('id'))]
                ship   = self.world.ships[int(intoNode.getNetTag('id'))]
                pos = ship.node.getPos()
                self.world.hit(ship,projectile)
            except KeyError:
                pass # projectiles or ships are already gone
                        
    