import direct.directbase.DirectStart
import math, sys, random, os
from pandac.PandaModules import *
from direct.showbase import DirectObject
from direct.task import Task

EXPLOSIONS = 'data/fx/explosions/'

SHOCKWAVE_CAMERA = False


#create reflection buffer for shockwave effect
if SHOCKWAVE_CAMERA:
    shockwaveBuffer=base.win.makeTextureBuffer("shockwave scene", 512, 512)
    shockwaveBuffer.setSort(-3)
    shockwaveBuffer.setClearColor(Vec4(0,0,0,0))
    
    shockwaveCamera=base.makeCamera(shockwaveBuffer, aspectRatio=base.getAspectRatio())
    shockwaveCamera.node().setCameraMask(BitMask32.bit(4))
    far = 10000.0
    normalLens = shockwaveCamera.node().getLens()
    normalLens.setFar(far)
    shockwaveCamera.node().setLens(normalLens)
    
    tempnode = NodePath(PandaNode("temp node"))
    shockwaveCamera.node().setInitialState(tempnode.getState())
    tempnode.remove()
    shockwaveCamera.reparentTo(base.camera)
    reflectionStage = TextureStage('reflection')
    reflectionStage.setSort(0)
    reflection = shockwaveBuffer.getTexture()

class AnimatedSprites:
    def __init__(self, world):
        self.world=world
        self.frames = {}
        self.prefix = {}
        self.textures = {}
        
        self.frames['bigsplo'] = 8
        self.prefix['bigsplo'] = ['A']
        #self.prefix['bigsplo'] = ['A', 'B', 'C']
        self.frames['fire'] = 16
        self.prefix['fire'] = ['A', 'B']
        self.frames['hitsplo'] = 3
        self.prefix['hitsplo'] = ['A']
        #self.prefix['hitsplo'] = ['A', 'B', 'C', 'D']
        #self.frames['regularsplo'] = 5
        #self.prefix['regularsplo'] = ['A', 'B', 'C', 'D', 'E', 'F']
        self.frames['smoke'] = 5
        self.prefix['smoke'] = ['B']
        #self.prefix['smoke'] = ['A', 'B', 'C']
        prefixList = []
        list = []
        
        for key in self.frames:
            del prefixList
            prefixList = []
            fps = self.frames[key]
            for n in xrange(0, len(self.prefix[key])):
                prefixList.append(self.prefix[key][n])
            for index in xrange(0, len(prefixList)):
                del list
                list = []
                for i in xrange(1, fps):
                    identifier = prefixList[index]
                    if i < 10: identifier = identifier+'0'
                    texture = loader.loadTexture(EXPLOSIONS+key+identifier+str(i)+'.png')
                    list.append(texture)
                texKey = key+identifier
                self.textures[texKey]=list
        print self.textures
        
    def playSmoke(self, object, fps=5):
##        findKey = random.randrange(1,3)
##        if findKey == 1: key='smokeA0'
##        if findKey == 2: key='smokeB0'
##        if findKey == 3: key='smokeC0'
        key='smokeB0'
        newTask = taskMgr.add(self.playStayTask, 'playStayTask')
        newTask.object = object
        newTask.fps = fps
        newTask.tex = self.textures[key]
        newTask.endframe = len(self.textures[key])-1
        
    def play(self, object, fps=15):
##        findKey = random.randrange(1,4)
##        if findKey == 1: key='hitsploA0'
##        if findKey == 2: key='hitsploB0'
##        if findKey == 3: key='hitsploC0'
##        if findKey == 4: key='hitsploD0'
        key = 'hitsploA0'
        lightIndex = self.world.lights.addDynamicLight(object, Vec4(20,50,2,1), 100, 0) 
        newTask = taskMgr.add(self.playTask, 'playTask')
        newTask.object = object
        newTask.fps = fps
        newTask.light = lightIndex
        newTask.tex = self.textures[key]
        newTask.endframe = len(self.textures[key])-1
        
    def playBigExplosion(self, object, fps=8):
##        findKey = random.randrange(1,3)
##        if findKey == 1: key='bigsploA0'
##        if findKey == 2: key='bigsploB0'
##        if findKey == 3: key='bigsploC0'
        key='bigsploA0'
        newTask = taskMgr.add(self.playBigExplosionTask, 'playBigExplosionTask')
        newTask.object = object
        newTask.fps = fps
        newTask.tex = self.textures[key]
        newTask.endframe = len(self.textures[key])-1
    
    def playFire(self, object, fps=30):
        findKey = random.randrange(1,2)
        if findKey == 1: key='fireA'
        if findKey == 2: key='fireB'
        
        newTask = taskMgr.add(self.playFireTask, 'playFireTask')
        newTask.object = object
        newTask.fps = fps
        newTask.tex = self.textures[key]
        newTask.endframe = len(self.textures[key])-1
        
    def playFireTask(self, task):
        currentFrame = int(task.time*task.fps)
        currentFrame = currentFrame%len(task.tex)
        task.object.setTexture(task.tex[currentFrame], 1)
        scale = 20 + task.time*10
        if task.endframe == currentFrame: 
            task.object.detachNode()
            task.object.remove()
            return task.done
        return task.cont
        
    def playBigExplosionTask(self, task):
        currentFrame = int(task.time*task.fps)
        currentFrame = currentFrame%len(task.tex)
        task.object.setTexture(task.tex[currentFrame], 1)
        if task.endframe == currentFrame: 
            task.object.detachNode()
            task.object.remove()
            return task.done
        return task.cont
        
    def playTask(self, task):
        
        currentFrame = int(task.time*task.fps)
        currentFrame = currentFrame%len(task.tex)
        task.object.setTexture(task.tex[currentFrame], 1)
        if task.endframe == currentFrame: 
            task.object.detachNode()
            task.object.remove()
            self.world.lights.removeDynamicLight(task.light) 
            return task.done
        return task.cont
    
    def playStayTask(self, task):
        
        currentFrame = int(task.time*task.fps)
        currentFrame = currentFrame%len(task.tex)
        task.object.setTexture(task.tex[currentFrame], 1)
        if task.endframe == currentFrame: 
            return task.done
        return task.cont

class Explosions:
    def __init__(self, world):
        self.world = world
        self.maxDecals = 50
        self.animation = AnimatedSprites(self.world)
        self.smokeDecals = []
        self.textureCard = loader.loadModel(EXPLOSIONS+'explosion.egg')
        self.smokeTexture = loader.loadTexture(EXPLOSIONS+'smokeA01.png')
        self.shockwaveModel = loader.loadModel(EXPLOSIONS+'sphere.bam')
        for index in xrange(0, self.maxDecals):
            newNode = self.instance()
            self.smokeDecals.append(newNode)
            self.smokeDecals[index].setTexture(self.smokeTexture, 1)
            self.smokeDecals[index].setBillboardPointEye()
            self.smokeDecals[index].setTransparency(TransparencyAttrib.MAlpha)
            self.smokeDecals[index].setScale(50)
            self.smokeDecals[index].setColor(1,1,1,1)
            self.smokeDecals[index].hide()
        self.decalIndex = 0
        
    def addCriticalDamageFx(self, ship):
        newTask = taskMgr.add(self.criticalDamageFxTask, 'criticalDamageFxTask')
        newTask.object = ship
        newTask.fps = 15
        newTask.frameSkip = 5

    def addHit(self, pos):
        newHitNode = self.instance()
        newHitNode.reparentTo(render)
        newHitNode.setScale(50)
        newHitNode.setTransparency(TransparencyAttrib.MAlpha)
        newHitNode.setBillboardPointEye()
        newHitNode.setPos(pos)
        self.animation.play(newHitNode)
        
    def addExplosion(self, ship, fps=30):
        #add explosionLight
        color = Vec4(200,100,200,10)
        fallOffMultiplyer = 100
        duration = 0
        lightIndex = self.world.lights.addDynamicLight(ship, color, fallOffMultiplyer, duration) 
        
        debris = []
        geomNodeCollection = ship.findAllMatches('**/*break*')
        for nodePath in geomNodeCollection.asList():
            name = nodePath.node().getName()
            if name == "": name="hull"
            newNode = GeomNode(name)
            geomNodes = nodePath.node()
            for i in range(geomNodes.getNumGeoms()):
                newNode.addGeom(geomNodes.modifyGeom(i))
                debris.append(render.attachNewNode(newNode))
            smokeSamples = []
            i = 0
            while 1:
                i +=1
                smokeSamples.append(self.decalIndex)
                self.decalIndex += 1
                if self.decalIndex > len(self.smokeDecals)-1: self.decalIndex = 0
                if i == 4: break
                
        newTask = taskMgr.add(self.mainExplosionTask, 'mainExplosionTask')
        newTask.debris = debris
        newTask.pos = ship.getPos()
        newTask.tmpTime = 0
        newTask.fps = fps
        newTask.primary = []
        newTask.decalIndex = smokeSamples
        newTask.vector = []
        newTask.light = lightIndex
        
    def criticalDamageFxTask(self, task):
        currentFrame = int(task.time*task.fps)
        currentFrame = currentFrame%task.frameSkip
        
        if currentFrame == 0:
            newFire = self.instance()
            newFire.reparentTo(render)
            newFire.setScale(50)
            try:
                pos = task.object.node.getPos()+Vec3(0,currentFrame,0)
                newFire.setPos(pos)
            except:
                newFire.detachNode()
                newFire.remove()
                return Task.done
            newFire.setTransparency(TransparencyAttrib.MAlpha)
            newFire.setBillboardPointEye()
            self.animation.playFire(newFire, 16)
        if task.object.structure < 20 or task.object.structure > 90:
            task.object.damageFx = 0
            return Task.done
        
        return Task.cont
        
    def mainExplosionTask(self, task):
        index = len(task.primary)
        if index < 1:
            task.primary.append(render.attachNewNode('primary0'))
            self.shockwaveModel.instanceTo(task.primary[0])
            task.primary[0].setScale(.001)
            task.primary[0].setPos(task.pos)
            if SHOCKWAVE_CAMERA:
                global reflection
                base.graphicsEngine.renderFrame() 
                reflection = shockwaveBuffer.getTexture()
                task.primary[0].setTexture(reflectionStage, reflection )
                
            task.primary[0].hide(BitMask32.bit(4))
            task.primary[0].setTransparency(TransparencyAttrib.MAlpha)
            task.primary[0].setTwoSided(1)
            
            task.primary.append(0)
            task.primary[1] = self.instance()
            task.primary[1].setScale(100)
            task.primary[1].setPos(task.pos)
            task.primary[1].setTransparency(TransparencyAttrib.MAlpha)
            task.primary[1].setBillboardPointEye()
            self.animation.playBigExplosion(task.primary[1])
            
            task.primary.append(0)
            task.primary[2] = self.instance()
            task.primary[2].setScale(100)
            pos = task.pos+Vec3(1,2,1)
            task.primary[2].setPos(pos)
            task.primary[2].setTransparency(TransparencyAttrib.MAlpha)
            task.primary[2].setBillboardPointEye()
            self.animation.playBigExplosion(task.primary[2])
            
            for i in xrange(0, len(task.decalIndex)):
                self.smokeDecals[task.decalIndex[i]].setPos(task.pos)
                self.smokeDecals[task.decalIndex[i]].show()
                self.animation.playSmoke(self.smokeDecals[task.decalIndex[i]])
            for i in xrange(0, len(task.debris)):
                task.debris[i].show()
                task.debris[i].setTransparency(TransparencyAttrib.MAlpha)
                task.debris[i].setPos(task.pos)
                x = random.randrange(-100, 100)
                y = random.randrange(-100,100)
                z = random.randrange(-100,100)
                task.vector.append(Vec3(x,y,z))
                task.vector[i].normalize()
            task.tmpTime = task.time
        else:
            currentTime = task.time - task.tmpTime
            alpha = 1.0 - (currentTime/2)
            alphaDebris = 1.0 - (currentTime/4)
            if alpha < 0: alpha = 0 
            if currentTime > 0.5 and currentTime < 0.7:
                task.primary[1] = self.instance()
                task.primary[1].setScale(200)
                task.primary[1].setPos(task.pos)
                task.primary[1].setTransparency(TransparencyAttrib.MAlpha)
                task.primary[1].setBillboardPointEye()
                self.animation.playBigExplosion(task.primary[1]) 
                
            if alphaDebris < 0: alphaDebris = 0
            for i in xrange(0, len(task.debris)):
                task.debris[i].setPos(task.debris[i], task.vector[i])
                task.debris[i].setColor(1,1,1,alphaDebris)
            for i in xrange(0, len(task.decalIndex)):
                self.smokeDecals[task.decalIndex[i]].setPos(self.smokeDecals[task.decalIndex[i]], task.vector[i]*alpha*0.04)
                
            task.primary[0].setScale(currentTime*100)
            task.primary[0].setColor(1,1,1,alpha)
            if alphaDebris == 0:
                task.primary[0].detachNode()
                task.primary[0].remove()
                for i in xrange(0, len(task.debris)):
                    task.debris[i].detachNode()
                    task.debris[i].remove()
                self.world.lights.removeDynamicLight(task.light)
                return Task.done
            
        return Task.cont
        
    def instance(self):
        geomNodeCollection = self.textureCard.findAllMatches('**/+GeomNode')
        explosionNode = render.attachNewNode('explosion-sprite-instance')
        
        for nodePath in geomNodeCollection.asList():
            name = nodePath.node().getName()
            if name == "": name="hull"
            newNode = GeomNode(name)
            geomNodes = nodePath.node()
            for i in range(geomNodes.getNumGeoms()):
                newNode.addGeom(geomNodes.modifyGeom(i))
                explosionNode.attachNewNode(newNode)
            explosionNode.hide(BitMask32.bit(4))
        return explosionNode