from random import *
import os
from direct.task import Task
from direct.showbase.DirectObject import DirectObject
import direct.directbase.DirectStart
from pandac.PandaModules import *
LEVELS = 'data/levels/'

class Level:
    
    COLLISION_SYSTEM = "eggoctree"
    TURN_OFF_LEVEL_MESH = False
    
    
    def __init__(self):
        self.levelNode = render.attachNewNode('level')
        self.geometryCache = {}
        self.currentLevel = ""
        self.levelNode.setTexture(loader.loadTexture("data/levels/metal.jpg"))
        
    def loadLevel(self, level):
        self.currentLevel = loader.loadModel(LEVELS+level+'.egg')

        self.currentLevel.ls()

       
        self.currentLevel.setBin("background",10)
        #self.currentLevel.setDepthWrite(False)
        #self.currentLevel.ls()

        
        self.currentLevel.reparentTo(self.levelNode)
        levelSpecificStaticMeshes = LEVELS+level+'-static/'
        
        geomNodeCollection = self.currentLevel.findAllMatches('**/inst*')
        
        for nodePath in geomNodeCollection.asList():
            instName = nodePath.node().getName()
            instName = instName.rsplit('-')
            modelPath = levelSpecificStaticMeshes+'sm-'+instName[1].strip()+'.egg'
            self.loadStaticMeshes(nodePath, modelPath)
            nodePath.hide()
            
        geomNodeCollection = self.currentLevel.findAllMatches('**/*spawn*')
        for nodePath in geomNodeCollection.asList():
            nodePath.hide()
            

      
        if self.COLLISION_SYSTEM == "octree":
            
            lowPolys = self.currentLevel.findAllMatches('**/+GeomNode')        
            assert lowPolys.asList() != [] 
            for nodePath in lowPolys.asList():     
                if "sec" in str(nodePath) and "low" in str(nodePath): 
                    print nodePath
                    fastNodePath = self.colOptimizer.optimize(nodePath)
                    fastNodePath.setTag('collisionType', 'brush')
                    fastNodePath.setCollideMask(BitMask32.bit(1))
                    fastNodePath.setPos(nodePath.getPos(self.currentLevel))
                    fastNodePath.reparentTo(self.currentLevel)
                    #fastNodePath.hide()
                    
            lowPolys = self.currentLevel.findAllMatches('**/*sec*low*')
            for nodePath in lowPolys.asList():
                nodePath.hide()
             
        elif self.COLLISION_SYSTEM == "eggoctree" : 

            collisoin = loader.loadModel(LEVELS+level+'_octree.egg')
            collisoin.reparentTo(self.currentLevel)
            collisoin.setTag('collisionType', 'brush')
            collisoin.setCollideMask(BitMask32.bit(1))
            collisoin.setColor(1,0,0,1)
           
            #collisoin.flattenStrong() 
                 
            lowPolys = self.currentLevel.findAllMatches('**/*sec*low*')
            for nodePath in lowPolys.asList():
                nodePath.hide()
         
        elif self.COLLISION_SYSTEM == "legion" :
            pass  
                
        elif self.COLLISION_SYSTEM == "panda":
            lowPolys = self.currentLevel.findAllMatches('**/*sec*low*')
            for nodePath in lowPolys.asList():
                nodePath.hide()
                nodePath.setCollideMask(BitMask32.bit(1))
                nodePath.setTag('collisionType', 'brush')
            
            
        if self.TURN_OFF_LEVEL_MESH:   
            lowPolys = self.currentLevel.findAllMatches('**/*sec*')
            for nodePath in lowPolys.asList():
                nodePath.hide()
                


            
    def loadStaticMeshes(self, nodePath, modelPath):
        if self.geometryCache.has_key(modelPath):
            newNode = self.levelNode.attachNewNode(modelPath)
            if self.geometryCache[modelPath] == None:
                #raise modelPath,"not found"
                return
            self.geometryCache[modelPath].instanceTo(newNode)
            newNode.setPos(nodePath.getPos())
            newNode.setHpr(nodePath.getHpr())
        else:
            self.geometryCache[modelPath] = loader.loadModel(modelPath)
            if self.geometryCache[modelPath] == None:
                #raise modelPath,"not found"
                return
            self.geometryCache[modelPath].setTag('collisionType', 'brush')
            #self.geometryCache[modelPath].findAllMatches('**/*low*').hide()
            #self.geometryCache[modelPath].findAllMatches('**/*low*').setCollideMask(BitMask32.bit(1))
            self.loadStaticMeshes(nodePath, modelPath)
            
            if self.COLLISION_SYSTEM == "octree":
              
                lowPolys = self.geometryCache[modelPath].findAllMatches('**/+GeomNode')        
                assert lowPolys.asList() != [] 
                for nodePath in lowPolys.asList():     
                    if "low" in str(nodePath): 
                        print nodePath
                        fastNodePath = self.colOptimizer.optimize(nodePath)
                        fastNodePath.setTag('collisionType', 'brush')
                        fastNodePath.setCollideMask(BitMask32.bit(1))
                        fastNodePath.setPos(nodePath.getPos(self.geometryCache[modelPath]))
                        fastNodePath.reparentTo(self.geometryCache[modelPath])
                        fastNodePath.hide()
                        
                lowPolys = self.geometryCache[modelPath].findAllMatches('**/*low*')
                for nodePath in lowPolys.asList():
                    nodePath.hide()
            
            elif self.COLLISION_SYSTEM == "legion" :
                pass  
            
            elif self.COLLISION_SYSTEM == "panda" or self.COLLISION_SYSTEM == "eggoctree":
                lowPolys = self.geometryCache[modelPath].findAllMatches('**/*low*')
                for nodePath in lowPolys.asList():
                    nodePath.hide()
                    nodePath.setCollideMask(BitMask32.bit(1))
                    nodePath.setTag('collisionType', 'brush')
            
            