import direct.directbase.DirectStart
import math, sys, random, os
from pandac.PandaModules import *
from direct.showbase import DirectObject
from direct.task import Task

class Lighting:
    def __init__(self, world):
        self.world = world
        self.ambientLight = AmbientLight( "ambientLight" )
        self.ambientLight.setColor( Vec4( .5, .5, .5, 1 ) )
        self.alnp = render.attachNewNode(self.ambientLight.upcastToPandaNode())
        render.setLight(self.alnp)
        self.lightNodes = []
        self.dynamicLights = {}
        self.index = 0
        
        
    def addSunLight(self, pos, color=Vec4(1,1,1,1),fallOffMultiplyer=1):
        index = len(self.lightNodes)
        sunLight=PointLight('sunlight'+str(index))
        sunLight.setAttenuation(Vec3(0, 0, .000001*fallOffMultiplyer))
        sunLight.setColor(color)
        self.lightNodes.append(render.attachNewNode(sunLight.upcastToPandaNode()))
        render.setLight(self.lightNodes[index])
        
    def addDynamicLight(self, attachNode, color=Vec4(1,1,1,1), fallOffMultiplyer=1, lifeTime=0):
        index = str(self.index)
        dynamicLight = PointLight('dynamicLight'+str(index))
        dynamicLight.setAttenuation( Vec3( 0, 0, .0001*fallOffMultiplyer ) )
        dynamicLight.setColor(color)
        self.dynamicLights[index] = attachNode.attachNewNode(dynamicLight.upcastToPandaNode())
        if lifeTime != 0:
            taskMgr.doMethodLater(lifeTime, self.removeDynamicLight, 'dynamicLifeTime'+index, extraArgs=[index])
        render.setLight(self.dynamicLights[index])
        self.index += 1
        return index
        
            
    def removeDynamicLight(self, index):
        render.clearLight(self.dynamicLights[index])
        del self.dynamicLights[index]