from level import *
from ship import *
from weapon import *
from direct.task import Task
from collisions import CollisionDetection
from time import time
from random import choice
from explosions import Explosions
from lighting import Lighting
from sounds import SoundFx

class World:
    
    def __init__(self,game):
        self.sounds = SoundFx()
        self.lights = Lighting(self)
        self.showCollisions = False
        self.lastTime = time()
        self.game = game
        self.level = Level()
        self.ships = {}
        self.projectiles = {}
        self.lastID = 0  
        self.dt = 0   
        self.collisionDetection = CollisionDetection(self)
        self.explosions = Explosions(self)
        self.lights.addSunLight(Vec3(0,100,0), Vec4(10,10,10,1), 100)
        self.lights.addSunLight(Vec3(50,-1500,0), Vec4(10,10,3,1), 100)
        
        
        
    def loadMap(self, map):
        self.level.loadLevel(map)
        spwanList = render.findAllMatches('**/*spawn*')
        self.spwanList = [spwanList[i].getPos() for i in range(spwanList.getNumPaths())]
        
    def addShip(self,ship):
        pos = choice(self.spwanList)
        ship.id = self.lastID
        ship.node.setPos(pos)
        ship.world = self
        self.lastID += 1
        self.ships[ship.id] = ship
        radius = ship.node.getBounds().getRadius()
        cSphere = CollisionSphere(0,0,0,radius)
        self.sounds.addSoundsToShip(ship)
        self.sounds.playEngine(ship)
        self.cNodePath = ship.node.attachNewNode(CollisionNode('collisionNode'))
        self.cNodePath.node().addSolid(cSphere)
        self.cNodePath.setTag('collisionType', 'shield')
        self.cNodePath.setTag('id', str(ship.id))
        self.collisionDetection.addFromNodePath(self.cNodePath)
        if self.showCollisions: self.cNodePath.show()

    def removeShip(self,ship):
        self.resetShip(ship)
        return
        self.sounds.removeSoundsFromShip(ship)
        del self.ships[ship.id]
        ship.node.remove()
        self.collisionDetection.traverser.removeCollider(self.cNodePath)
        
    def resetShip(self,ship):
        ship.shield = ship.maxShield
        ship.armor = ship.maxArmor
        ship.structure = ship.maxStructure
        ship.node.setPos(choice(self.spwanList))
        ship.pilot.reset()
        
    def hitWorld(self,projectile):
        self.sounds.playWorldHit(projectile.launched, projectile.node.getPos())
        self.explosions.addHit(projectile.node.getPos())
        
        
    def fire(self,ship,projectile):
        projectile.id = self.lastID
        self.lastID += 1
        projectile.node.setPos(ship.node.getPos())
        projectile.node.setHpr(ship.node.getHpr())
        projectile.launched = ship
        self.projectiles[projectile.id] = projectile 
        
        self.sounds.playFire(ship, 0.2)
        #self.lights.addDynamicLight(projectile.node, Vec4(1,.5,2,1), 1,4)
        
        radius = projectile.node.getBounds().getRadius()
        #cSphere = CollisionSphere(0,0,0,radius  )
        cSphere = CollisionSegment(0,projectile.speed/8,0,0,0,0)
        
        self.cNodePath = projectile.node.attachNewNode(CollisionNode('collisionNode'))
        self.cNodePath.node().addSolid(cSphere)
        if self.showCollisions: self.cNodePath.show()
        self.cNodePath.setTag('collisionType', 'weapon')
        self.cNodePath.setTag('id', str(projectile.id))
        self.collisionDetection.addFromNodePath(self.cNodePath)
    
    def hit(self,ship,projectile):
        if projectile.launched != ship:
            
            if ship.shield > 0 :
                ship.shield -= projectile.damage
            else:
                if ship.armor > 0 :
                    ship.armor -= projectile.damage
                else:
                    ship.structure -= projectile.damage
            
            self.sounds.playHit(ship)
            #self.explode(projectile)
            self.explosions.addHit(projectile.node.getPos())

            self.removeProjectile(projectile)
            if ship.structure < 50 and ship.damageFx ==0:
                self.explosions.addCriticalDamageFx(ship)
                ship.damageFx = 1
            if ship.structure <= 0 :
                ship.damageFx = 0
                self.sounds.playExplosion(ship, 0.8)
                self.explosions.addExplosion(ship.node)
                self.removeShip(ship)
                projectile.launched.frags += 1
                
    
    def explode(self,projectile):
        explosion = Explosion() 
        explosion.id = self.lastID
        self.lastID += 1
        explosion.node.setPos(projectile.node.getPos())
        explosion.node.setHpr(projectile.node.getHpr())
        self.projectiles[explosion.id] = explosion 
            
   
    
    def removeProjectile(self,projectile):
        del self.projectiles[projectile.id]
        projectile.node.remove()
        self.collisionDetection.traverser.removeCollider(self.cNodePath)
        
    def simulate(self,task):
        
        now = time()
        dt = (now - self.lastTime)
        self.dt = self.dt*.9 + dt*.1 
        if self.dt > .2:
            self.dt = .2
        self.lastTime = now 
                
        self.simulateShips()
        self.simulateWeapons()
        return Task.cont
        
    def simulateShips(self):
        for ship in self.ships.values():
            ship.simulate(self.dt)
    
    def simulateWeapons(self):
        
        for projectile in self.projectiles.values():
            projectile.simulate(self.dt)
            projectile.time += self.dt
            if projectile.time > projectile.timeOut:
               self.removeProjectile(projectile)
